# -*- coding: utf-8 -*-
from __future__ import absolute_import

from bitstring import BitArray
from oset import oset

from .basescrambler import BaseScrambler


class MultiScrambler(BaseScrambler):
    """
    MultiScrambler is an extended version of BaseScrambler that allows for
    scrambling data multiple times at different-sized chunks for improved
    cryptographic qualities.
    """
    def __init__(self, trans, keys):
        """
        Where trans is an array of integers specifying the size of chunks that
        the data should be split into for each iteration of scrambling.

        Where keys is an array of oset objects that specifies the order each
        iteration of scrambling should be re-arranged into.
        """
        if not hasattr(trans, '__iter__'):
            raise TypeError('Non-iterable type given for trans argument.')
        else:
            for tran in trans:
                if not isinstance(tran, int):
                    raise TypeError('Expected int for item in trans array.')
        if not hasattr(keys, '__iter__'):
            raise TypeError('Non-iterable type given for keys argument')
        else:
            for key in keys:
                if not isinstance(key, oset):
                    raise TypeError(('Expected oset object for item in keys '
                                     'array'))
        self.trans = trans
        self.keys = keys

    def scramble(self, data):
        """
        Where data is a bytearray object, split into chunks the size of
        self.trans, re-arrange according to self.keys order then convert back
        to bytearray and return this. Do this for each item in trans array and
        keys array.
        """
        if not isinstance(data, bytearray):
            raise TypeError(('Expected bytearray object for data argument, '
                             'received {}').format(type(data)))
        # convert to BitArray for easier chunking
        clear = BitArray(data)
        code = BitArray(clear)
        for index, tran in enumerate(self.trans):
            # get the key for this turn
            key = self.keys[index]
            code = super(MultiScrambler, self)._scramble(code, tran, key)
        return bytearray(code.tobytes())

    def unscramble(self, data):
        """
        Where data is a bytearray object, split into chunks the size of
        self.trans, re-arrange according to self.keys order then convert back
        to bytearray and return this. Do this for each item in trans array and
        keys array.
        """
        if not isinstance(data, bytearray):
            raise TypeError(('Expected bytearray object for data argument, '
                             'received {}').format(type(data)))
        # convert to BitArray for easier chunking
        code = BitArray(data)
        clear = BitArray(code)
        for index, tran in reversed(list(enumerate(self.trans))):
            # get the key for this turn
            key = self.keys[index]
            clear = super(MultiScrambler, self)._unscramble(clear, tran, key)
        return bytearray(clear.tobytes())



# -*- coding: utf-8 -*-
from __future__ import absolute_import

import bitstring


def bits_to_group(array, size):
    """
    Convert a bitstring.BitArray or array of individual bits to an array
    of groups of bits of a given size.

    Raises exception if the length of the array is not a multiple of the
    given group size.
    """
    # first it it's an array of bitarrays then assemble items into one bitarray
    if hasattr(array, '__iter__'):
        bits = bitstring.BitArray()
        for item in array:
            bits += item
    else:
        bits = bitstring.BitArray(array)
    # sanity checking
    if len(bits) % size != 0:
        raise ValueError('The given BitArray or array instance size is not '
                         'a multiple of the given group size.')
    result = []
    for index in range(len(bits))[::size]:
        result.append(bits[index:index+size])
    return result



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import absolute_import

from setuptools import setup

setup(
    name='pyscrambler',
    version='0.0.11',
    description='Concept data scrambler using permutations as a basis',
    url='https://github.com/saxbophone/pyscrambler',
    author='Joshua Saxby',
    author_email='joshua.a.saxby@gmail.com',
    license='MIT',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Intended Audience :: Education',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Topic :: Security :: Cryptography'
    ],
    packages=['pyscrambler', 'pyscrambler.scramblers'],
    install_requires=['bitstring==3.1.3',
                      'oset==0.1.3'],
    extras_require={
        'test': ['coverage==3.7.1'],
    },
    zip_safe=False
)



# -*- coding: utf-8 -*-
from __future__ import absolute_import

from . import binary
from . import permutation
from . import rearrange
from . import scramblers


__all__ = ['permutation', 'rearrange', 'binary', 'scramblers']



# -*- coding: utf-8 -*-
from __future__ import absolute_import

from bitstring import BitArray
from oset import oset

from ..binary import bits_to_group
from ..rearrange import re_arrange, reverse_order


class BaseScrambler(object):
    """
    BaseScrambler is the base class for creating different scrambling systems
    using this module.
    """
    def __init__(self, trans, key):
        """
        Where trans is an integer specifying the size of chunks (in bits) that
        the input data should be broken into before re-ordering the data.
        Where key is an oset (ordered set) object containing the desired order
        that the data should be re-arranged into.
        """
        if isinstance(trans, int):
            self.trans = trans
        else:
            raise TypeError(
                'Expected int for trans argument, received {}'.format(
                    type(trans)))
        if isinstance(key, oset):
            self.keys = key
        else:
            raise TypeError(
                'Expected oset object for key argument, received {}'.format(
                    type(key)))

    def _scramble(self, clear, width, key):
        """
        Lower-level plumbing method used by the scramble function.

        This accepts clear as a BitArray, width as an integer and key as any
        array-type and re-orders the data after splitting it into chunks of
        given size.

        This is here to make it easier to override this class.
        """
        # get length in bits
        size = len(clear)
        # split up into chunks of size given by trans
        chunks = bits_to_group(clear, width)
        # re-arrange these chunks according to key
        jumbled = re_arrange(chunks, key)
        # convert back to one BitArray (get first and only item out of list)
        code = bits_to_group(jumbled, size)[0]
        return code

    def scramble(self, data):
        """
        Where data is a bytearray object, split into chunks the size of
        self.trans, re-arrange according to self.keys order then convert back
        to bytearray and return this.
        """
        if not isinstance(data, bytearray):
            raise TypeError(('Expected bytearray object for data argument, '
                             'received {}').format(type(data)))
        # convert to BitArray for easier chunking
        clear = BitArray(data)
        code = self._scramble(clear, self.trans, self.keys)
        return bytearray(code.tobytes())

    def _unscramble(self, code, width, key):
        """
        Lower-level plumbing method used by the unscramble function.

        This accepts code as a BitArray, width as an integer and key as any
        array-type and re-orders the data after splitting it into chunks of
        given size.

        This is here to make it easier to override this class.
        """
        # get length in bits
        size = len(code)
        # split up into chunks of size given by trans
        chunks = bits_to_group(code, width)
        # get the inverse of the key
        i_key = reverse_order(key)
        # re-arrange these chunks according to key
        jumbled = re_arrange(chunks, i_key)
        # convert back to one BitArray (get first and only item out of list)
        clear = bits_to_group(jumbled, size)[0]
        return clear

    def unscramble(self, data):
        """
        Where data is a bytearray object, split into chunks the size of
        self.trans, re-arrange according to self.keys inverse order then
        convert back to bytearray and return this. (Achieves the opposite of
        scramble method).
        """
        if not isinstance(data, bytearray):
            raise TypeError(('Expected bytearray object for data argument, '
                             'received {}').format(type(data)))
        # convert to BitArray for easier chunking
        code = BitArray(data)
        clear = self._unscramble(code, self.trans, self.keys)
        return bytearray(clear.tobytes())
